import binascii

# Assuming yaraforge.metadata.pathnames and yaraforge.utils.logger are valid modules
from yaraforge.metadata import pathnames
from yaraforge.utils.logger import get_global_logger

logger = get_global_logger(pathnames['logger_dir'])


def format_hex(data):
    """
    Format the hex data
    :param data:
    :return: Formatted hex data
    """
    return " ".join(data[i:i + 2] for i in range(0, len(data), 2))


def _wildcard_bytes(data, offset, length):
    """
    Wildcard the bytes in the data
    :param data:
    :param offset:
    :param length:
    :return: Wildcarded bytes
    """
    for i in range(offset, offset + length):
        data[i] = "?"
    return data


def _process_instruction(ins):
    """
    Process the instruction
    :param ins:
    :return: Processed instruction
    """
    ins_hex_list = list(binascii.hexlify(ins.bytes).decode("ascii").upper())

    ins_hex_list = _wildcard_bytes(ins_hex_list, ins.disp_offset * 2, ins.disp_size * 2)

    signature = ''.join(ins_hex_list)
    return signature


def format_bytes_with_space(bytes_str):
    """將機器碼字串中每兩個字符之間加入空格"""
    return ' '.join(bytes_str[i:i + 2] for i in range(0, len(bytes_str), 2))



import logging
import os
from datetime import datetime
from pathlib import Path

from metadata import pathnames, metadata

logger = None


def initialize_logger():
    """
    Initialize the logger
    :return: None
    """
    try:
        # Use the logger_dir defined in the pathnames dictionary
        log_dir = pathnames['logger_dir']
        # Ensure the log directory exists
        Path(log_dir).mkdir(parents=True, exist_ok=True)

        # Create the log file name
        log_filename = datetime.now().strftime("%Y%m%d_%H%M%S") + ".log"
        log_filepath = os.path.join(log_dir, log_filename)

        # Set up logging configuration
        logging.basicConfig(
            filename=log_filepath,
            level=logging.INFO,
            format=f'%(asctime)s - {metadata["plugin_name"]} - [%(filename)s] - %(levelname)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
    except Exception as e:
        print(f"Error occurred while initializing logger: {e}")


def get_global_logger(log_dir=pathnames['logger_dir'], name='yaraforge', level=logging.INFO):
    """
    Get the global logger
    :param log_dir:
    :param name:
    :param level:
    :return:
    """
    global logger
    try:
        if logger is None:
            # Ensure the log directory exists
            os.makedirs(log_dir, exist_ok=True)
            datetime_str = datetime.now().strftime("%Y%m%d_%H%M%S")
            log_file_name = f"{datetime_str}.log"
            log_file_path = Path(log_dir) / log_file_name

            logger = logging.getLogger(name)
            logger.setLevel(level)
            logger.propagate = False  # Turn off propagation

            if not logger.handlers:
                # Create a file handler and formatter, then add them to the logger
                handler = logging.FileHandler(log_file_path)
                formatter = logging.Formatter(
                    f'%(asctime)s - {metadata["plugin_name"]} - [%(filename)s] - %(levelname)s '
                    f'- %(message)s')
                handler.setFormatter(formatter)
                logger.addHandler(handler)

        return logger
    except Exception as e:
        print(f"Error occurred while obtaining global logger: {e}")
        return None  # Perhaps return a default logger or None, depending on the situation





import json
from pathlib import Path

from yaraforge.metadata import pathnames
from yaraforge.utils.common import get_instructions_for_address
from yaraforge.utils.logger import get_global_logger

logger = get_global_logger(pathnames['logger_dir'])


class InstructionMaker:
    def __init__(self, file_hex_md5):
        """
        Initialize the InstructionMaker object.
        :param file_hex_md5: The MD5 hash of the file.
        :return: None
        """
        self.file_hex_md5 = file_hex_md5
        self.instructions = []

    def make_instructions(self):
        """
        Make instructions for the file.

        :return: None
        """
        pretty_dump_file_path = pathnames['pretty_dump_dir'] / f"{self.file_hex_md5}_pretty_dump.json"
        if pretty_dump_file_path.is_file():
            with open(pretty_dump_file_path, 'r', encoding='utf-8') as file:
                pretty_dump_data = json.load(file)
        else:
            logger.error(f"Pretty dump file {pretty_dump_file_path} not found.")
            return self

        if pretty_dump_data and 'rules' in pretty_dump_data:
            for rule_name, rule_details in pretty_dump_data['rules'].items():
                if 'matches' in rule_details:
                    for match_list in rule_details['matches']:
                        for match in match_list:
                            if match.get('type') == 'absolute':
                                match_value = match.get('value', 0)
                                match_value_hex = hex(int(match_value)) if match_value else '0x0'
                                instructions = get_instructions_for_address(match_value)
                                self.instructions.append({
                                    'Rule Name': rule_name,
                                    'Match Type': match.get('type', ''),
                                    'Value': match_value,
                                    'Address': match_value_hex,
                                    'Instructions': instructions
                                })

        # 在函數末尾加上保存指令的邏輯
        if not self.instructions:
            logger.error("No instructions to save.")
            return

        file_path = Path(pathnames['instructions_dir']) / f"{self.file_hex_md5}_instructions.json"
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(self.instructions, f, indent=4, ensure_ascii=False)
        logger.info(f"Instructions has been saved to {file_path}")
        return self




import json
from datetime import datetime
from ..metadata import *

import idaapi
import idc
from capstone import *

from ..utils.opcode_processor import *
from ..utils.opcode_processor import _process_instruction
from ..version import get_version

logger = get_global_logger(pathnames['logger_dir'])


class YaraMaker:
    def __init__(self, file_hex_md5):
        """
        Initialize the YaraMaker object.
        :param file_hex_md5: The MD5 hash of the file.
        :return: None
        """
        self.file_hex_md5 = file_hex_md5
        self.sig_mode = "normal"
        self.strings = []
        self.comments = []
        self.metas = {
            "generated_by": f"{metadata['plugin_name']} v{get_version()}",
            "date": "\"{}\"".format(datetime.now().strftime("%Y-%m-%d %H:%M")),
            "version": f"{get_version()}",
            "hash": f"{file_hex_md5}",
        }
        self.output_dir = Path(pathnames['yara_rules_dir']) / file_hex_md5
        self.output_dir.mkdir(parents=True, exist_ok=True)
        self.instructions_info = []

    def generate_rule(self):
        """
        Generate YARA rules for the file.
        :return: None
        """
        instructions_path = Path(pathnames['instructions_dir']) / f"{self.file_hex_md5}_instructions.json"

        with open(instructions_path, 'r', encoding='utf-8') as file:
            data = json.load(file)

        for rule in data:
            address = int(rule.get("Address"), 16)
            rule_name = rule.get("Rule Name", f"generated_rule_{hex(address)}").replace(' ', '_')

            self.strings = []  # Reset strings list for each rule
            self.comments = []  # Reset comments list for each rule
            self.instructions_info = []  # Reset instructions_info list for each rule
            all_instructions_signature = []
            func = idaapi.get_func(address)
            if not func:
                print(f"Address {hex(address)} does not belong to any function.")
                continue

            md = Cs(CS_ARCH_X86, CS_MODE_64)
            md.detail = True

            fci = idaapi.FlowChart(func)
            for block in fci:
                if block.start_ea <= address < block.end_ea:
                    code_bytes = idc.get_bytes(block.start_ea, block.end_ea - block.start_ea)
                    disasm = md.disasm(code_bytes, block.start_ea)
                    for ins in disasm:
                        signature = _process_instruction(ins)
                        formatted_signature = format_hex(signature)
                        all_instructions_signature.append(formatted_signature)
                        bytes_formatted = format_bytes_with_space(ins.bytes.hex().upper())
                        instruction_detail = {
                            'address': ins.address,
                            'mnemonic': ins.mnemonic,
                            'op_str': ins.op_str,
                            'bytes': bytes_formatted
                        }
                        self.instructions_info.append(instruction_detail)

            yara_path = Path(pathnames['yara_rules_dir']) / f"{self.file_hex_md5}"
            if all_instructions_signature:
                signature_str = "\n\t\t\t".join(all_instructions_signature)
                self.print_rule(rule_name, address, signature_str)
        print(f"YARA rules has been saved to {yara_path}")

    def print_rule(self, rule_name, address, signature_str):
        """
        Print the YARA rule to a file.
        :param rule_name:
        :param address:
        :param signature_str:
        :return:
        """
        formatted_rule_name = f"{rule_name}_{hex(address)}"
        rule_comments = "\n\t/*\n\t" + "\n\t".join([
            f"{hex(ins['address'])}:\t{ins['bytes'].ljust(40)} ; {ins['mnemonic'].ljust(10)}  {ins['op_str']}"
            for ins in self.instructions_info
        ]) + "\n\t*/\n"

        rule_content = f"rule {formatted_rule_name} {{\n"
        rule_content += "  meta:\n"
        for key, value in self.metas.items():
            rule_content += f"    {key} = {value}\n"
        rule_content += rule_comments
        rule_content += "  strings:\n"
        rule_content += f"    $chunk_1 = {{\n\t\t\t{signature_str}\n\t\t}}\n"
        rule_content += "  condition:\n    any of them\n}\n"

        yara_rule_path = self.output_dir / f"{formatted_rule_name}.yar"
        with open(yara_rule_path, 'w', encoding='utf-8') as file:
            file.write(rule_content)
        logger.info(f"YARA rule for {formatted_rule_name} has been saved to {yara_rule_path}")








from setuptools import setup, find_packages

from metadata import metadata
from version import get_version


setup(
    name=f"{metadata['plugin_name']}",
    version=f"{get_version()}",
    author=", ".join([author['name'] for author in metadata['authors']]),
    author_email=", ".join([author['email'] for author in metadata['authors']]),
    description=f"{metadata['description']}",
    url=f"{metadata['github_url']}",
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.9',
        'Development Status :: 4 - Beta',
    ],
    python_requires=f"{metadata['python_requires']}",
    install_requires=[
        'capstone>=5.0.1',
        'flare-capa>=7.0.1',
    ],
    test_suite='yaraforge.tests',
)





from pathlib import Path

import ida_kernwin
import shutil
import os

from yaraforge.metadata import pathnames
from yaraforge.utils.common import get_desktop_path
from yaraforge.utils.logger import get_global_logger

logger = get_global_logger(pathnames['logger_dir'])
class DumpAsker:
    def __init__(self):
        """
        Initialize the DumpAsker object.

        :return: None
        """
        self.yaraforge_dir_path = Path(pathnames['yaraforge_dir'])
        self.desktop_path = get_desktop_path()

    def ask_user_for_dump(self):
        """
        Ask the user if they want to dump the caches.
        :return: None
        """
        answer = ida_kernwin.ask_yn(ida_kernwin.ASKBTN_NO, "Dump Caches on desktop?")
        if answer == ida_kernwin.ASKBTN_YES:
            target_path = os.path.join(self.desktop_path, os.path.basename(self.yaraforge_dir_path))
            try:
                # 檢查目標路徑是否存在，如果存在則刪除
                if os.path.exists(target_path):
                    shutil.rmtree(target_path)
                shutil.copytree(self.yaraforge_dir_path, target_path)
                print(f"Cache directory successfully copied to {target_path}")
            except Exception as e:
                print(f"Error copying cache directory: {e}")
        else:
            print("User chose not to dump caches.")
